/*
 * $Header: /home/cvs/jakarta-slide/src/share/org/apache/slide/store/ContentStore.java,v 1.4 2001/01/12 17:42:22 juergen Exp $
 * $Revision: 1.4 $
 * $Date: 2001/01/12 17:42:22 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.slide.store;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Date;
import java.io.FileWriter;
import java.io.IOException;
import org.apache.slide.structure.*;
import org.apache.slide.common.*;
import org.apache.slide.content.*;

/**
 * Store service.
 *
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.4 $
 */
public interface ContentStore extends Service {
    
    
    // ------------------------------------------------------ Interface Methods
    
    
    /**
     * Retrieve revision content.
     *
     * @param uri Uri
     * @param revisionNumber Node revision number
     */
    NodeRevisionContent retrieveRevisionContent
        (Uri uri, NodeRevisionDescriptor revisionDescriptor)
        throws ServiceAccessException, RevisionNotFoundException;
    
    
    /**
     * Create revision content.
     * <p>
     * Notes:
     * <ul>
     * <li>If a content length is specified by the revisionDescriptor
     * (ie, revisionDescriptor.getContentLength != -1), then
     * the ContentStore MUST enforce it, and throw a ServiceAccessException
     * if an incorrect number of bytes are read. It MUST also revert any
     * changes made to the unedrlying repository.</li>
     * <li>If a content length is not specified by the revisionDescriptor
     * (ie, revisionDescriptor.getContentLength == -1), then the ContentStore
     * MUST read all the bytes available and then call
     * revisionDescriptor.setContentLength(numberOfBytesRead).</li>
     * </ul>
     *
     * @param uri Uri
     * @param revisionDescriptor Node revision descriptor
     * @param revisionContent Node revision content
     */
    void createRevisionContent
        (Uri uri, NodeRevisionDescriptor revisionDescriptor,
         NodeRevisionContent revisionContent)
        throws ServiceAccessException, RevisionAlreadyExistException;
    
    
    /**
     * Modify revision content.
     * <p>
     * Notes:
     * <ul>
     * <li>If a content length is specified by the revisionDescriptor
     * (ie, revisionDescriptor.getContentLength != -1), then
     * the ContentStore MUST enforce it, and throw a ServiceAccessException
     * if an incorrect number of bytes are read. It MUST also revert any
     * changes made to the unedrlying repository.</li>
     * <li>If a content length is not specified by the revisionDescriptor
     * (ie, revisionDescriptor.getContentLength == -1), then the ContentStore
     * MUST read all the bytes available and then call
     * revisionDescriptor.setContentLength(numberOfBytesRead).</li>
     * </ul>
     *
     * @param uri Uri
     * @param revisionDescriptor Node revision descriptor
     * @param revisionContent Node revision content
     */
    void storeRevisionContent
        (Uri uri, NodeRevisionDescriptor revisionDescriptor,
         NodeRevisionContent revisionContent)
        throws ServiceAccessException, RevisionNotFoundException;
    
    
    /**
     * Remove revision content.
     *
     * @param uri Uri
     * @param revisionNumber Node revision number
     */
    void removeRevisionContent(Uri uri, NodeRevisionDescriptor revisionDescriptor)
        throws ServiceAccessException;
    
}
