/*
 * $Header: /home/cvs/jakarta-slide/src/stores/slidestore/reference/MemoryDescriptorsStore.java,v 1.7 2001/07/21 13:11:15 dirkv Exp $
 * $Revision: 1.7 $
 * $Date: 2001/07/21 13:11:15 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package slidestore.reference;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.io.FileWriter;
import java.io.IOException;
import org.apache.slide.common.*;
import org.apache.slide.store.*;
import org.apache.slide.structure.*;
import org.apache.slide.security.*;
import org.apache.slide.lock.*;
import org.apache.slide.content.*;

/**
 * Memory only (no persistence) DescriptorsStore implementation.
 * Reference implementation.
 *
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.7 $
 */
public class MemoryDescriptorsStore extends AbstractService
    implements LockStore, NodeStore, RevisionDescriptorsStore,
    RevisionDescriptorStore, SecurityStore {
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Objects hashtable.
     */
    private Hashtable objects;
    
    
    /**
     * Permissions hashtable.
     */
    private Hashtable permissions;
    
    
    /**
     * Locks hashtable.
     */
    private Hashtable locks;
    
    
    /**
     * Revision descriptors hashtable.
     */
    private Hashtable descriptors;
    
    
    /**
     * Revision descriptor hashtable.
     */
    private Hashtable descriptor;
    
    
    // ---------------------------------------------------- ServiceImpl Methods
    
    
    /**
     * Initializes the descriptors store with a set of parameters.
     * Those could be :
     * <li>User name, login info
     * <li>Host name on which to connect
     * <li>Remote port
     * <li>JDBC driver whoich is to be used :-)
     * <li>Anything else ...
     *
     * @param parameters Hashtable containing the parameters' name
     * and associated value
     */
    public void setParameters(Hashtable parameters)
        throws ServiceParameterErrorException,
        ServiceParameterMissingException {
    }
    
    
    /**
     * Connects to descriptors store.
     *
     * @exception DataException
     */
    public synchronized void connect()
        throws ServiceConnectionFailedException {
        if (objects == null) {
            objects = new Hashtable();
            permissions = new Hashtable();
            locks = new Hashtable();
            descriptors = new Hashtable();
            descriptor = new Hashtable();
        }
    }
    
    
    /**
     * Disconnects from descriptors store.
     *
     * @exception ServiceDisconnectionFailedException
     */
    public void disconnect()
        throws ServiceDisconnectionFailedException {
    }
    
    
    /**
     * Initializes descriptors store.
     *
     * @exception ServiceInitializationFailedException Throws an exception
     * if the descriptors store has already been initialized before
     */
    public synchronized void initialize(NamespaceAccessToken token)
        throws ServiceInitializationFailedException {
        super.initialize(token);
        objects = new Hashtable();
        permissions = new Hashtable();
        locks = new Hashtable();
        descriptors = new Hashtable();
        descriptor = new Hashtable();
    }
    
    
    /**
     * Deletes descriptors store. Should remove stored data if possible.
     *
     * @exception ServiceResetFailedException Reset failed
     */
    public synchronized void reset()
        throws ServiceResetFailedException {
            
        objects = new Hashtable();
        permissions = new Hashtable();
        locks = new Hashtable();
        descriptors = new Hashtable();
        descriptor = new Hashtable();
    }
    
    
    /**
     * Does nothing.
     *
     * @exception ServiceAccessException Service access error
     */
    public void commit()
        throws ServiceAccessException {
    }
    
    
    /**
     * This function tells whether or not the descriptors store is connected.
     *
     * @return boolean true if we are connected
     * @exception ServiceAccessException Service access error
     */
    public boolean isConnected()
        throws ServiceAccessException {
        return true;
    }
    
    
    /**
     * Idicates whether or not the objects managed by this service should be
     * cached. Caching is enabled by default.
     *
     * @return boolean True if results should be cached
     */
    public boolean cacheResults() {
        return false;
    }
    
    
    // ----------------------------------------------- DescriptorsStore Methods
    
    
    /**
     * Retrive an object from the Descriptors Store.
     *
     * @param uri Uri of the object we want to retrieve
     * @exception ServiceAccessException Error accessing the Descriptors Store
     * @exception ObjectNotFoundException The object to retrieve was not found
     */
    public ObjectNode retrieveObject(Uri uri)
        throws ServiceAccessException, ObjectNotFoundException {
        Object tempObject = objects.get(uri.toString());
        if (tempObject == null) {
            throw new ObjectNotFoundException(uri);
        }
        return ((ObjectNode) tempObject).cloneObject();
    }
    
    
    /**
     * Store an object in the Descriptors Store.
     *
     * @param object Object to update
     * @exception ServiceAccessException Error accessing the Descriptors Store
     * @exception ObjectNotFoundException The object to update was not found
     */
    public void storeObject(Uri uri, ObjectNode object)
        throws ServiceAccessException, ObjectNotFoundException {
        // We should also do a serialization to the filesystem
        objects.put(uri.toString(), object.cloneObject());
    }
    
    
    /**
     * Create a new object in the Descriptors Store.
     *
     * @param object SlideObject
     * @param uri Uri of the object we want to create
     * @exception ServiceAccessException Error accessing the Descriptors Store
     * @exception ObjectAlreadyExistsException An object already exists
     * at this Uri
     */
    public void createObject(Uri uri, ObjectNode object)
        throws ServiceAccessException, ObjectAlreadyExistsException {
        try {
            if (objects.containsKey(uri.toString())) {
                throw new ObjectAlreadyExistsException(uri.toString());
            }
            storeObject(uri, object);
        } catch(ObjectNotFoundException e) {
            // Never happens
        }
    }
    
    
    /**
     * Remove an object from the Descriptors Store.
     *
     * @param object Object to remove
     * @exception ServiceAccessException Error accessing the Descriptors Store
     * @exception ObjectNotFoundException The object to remove was not found
     */
    public void removeObject(Uri uri, ObjectNode object)
        throws ServiceAccessException, ObjectNotFoundException {
        if (!objects.containsKey(uri.toString())) {
            throw new ObjectNotFoundException(uri);
        }
        objects.remove(uri.toString());
    }
    
    
    /**
     * Store an object permissions in the Descriptors Store.
     *
     * @param permission Permission we want to create
     * @exception ServiceAccessException Error accessing the Descriptors Store
     */
    public void grantPermission(Uri uri, NodePermission permission)
        throws ServiceAccessException {
        Object value = permissions.get(uri.toString());
        Vector permissionsVector = null;
        if (value == null) {
            permissionsVector = new Vector();
            permissions.put(uri.toString(), permissionsVector);
        } else {
            permissionsVector = (Vector) value;
        }
        permissionsVector.addElement(permission.cloneObject());
    }
    
    
    /**
     * Store an object permissions in the Descriptors Store.
     *
     * @param permission Permission we want to create
     * @exception ServiceAccessException Error accessing the Descriptors Store
     */
    public void revokePermission(Uri uri, NodePermission permission)
        throws ServiceAccessException {
        Object value = permissions.get(uri.toString());
        Vector permissionsVector = null;
        if (value != null) {
            permissionsVector = (Vector) value;
            permissionsVector.removeElement(permission);
        }
        
    }
    
    
    /**
     * Revoke all the permissions on the object .
     *
     * @param permission Permission we want to create
     * @exception ServiceAccessException Error accessing the Descriptors Store
     */
    public void revokePermissions(Uri uri)
        throws ServiceAccessException {
        Object value = permissions.get(uri.toString());
        Vector permissionsVector = null;
        if (value != null) {
            permissionsVector = (Vector) value;
            permissionsVector.removeAllElements();
        }
    }
    
    
    /**
     * Store an object permissions in the Descriptors Store.
     *
     * @param permission Permission we want to create
     * @exception ServiceAccessException Error accessing the Descriptors Store
     */
    public Enumeration enumeratePermissions(Uri uri)
        throws ServiceAccessException {
        Object value = permissions.get(uri.toString());
        Vector permissionsVector = null;
        if (value != null) {
            permissionsVector = (Vector) value;
        } else {
            permissionsVector = new Vector();
        }
        return permissionsVector.elements();
    }
    
    
    /**
     * Puts a lock on a subject.
     *
     * @param lock Lock token
     * @exception ServiceAccessException Service access error
     */
    public void putLock(Uri uri, NodeLock lock)
        throws ServiceAccessException {
        Object value = locks.get(uri.toString());
        Vector locksVector = null;
        if (value == null) {
            locksVector = new Vector();
            locks.put(uri.toString(), locksVector);
        } else {
            locksVector = (Vector) value;
        }
        locksVector.addElement(lock.cloneObject());
    }
    
    
    /**
     * Renews a lock.
     *
     * @param lock Token to renew
     * @exception ServiceAccessException Service access error
     * @exception LockTokenNotFoundException Lock token was not found
     */
    public void renewLock(Uri uri, NodeLock lock)
        throws ServiceAccessException, LockTokenNotFoundException {
        Object value = locks.get(uri.toString());
        Vector locksVector = null;
        if (value == null) {
            throw new LockTokenNotFoundException(lock);
        } else {
            locksVector = (Vector) value;
            boolean wasPresent = locksVector.removeElement(lock);
            if (!wasPresent) {
                throw new LockTokenNotFoundException(lock);
            }
            locksVector.addElement(lock.cloneObject());
        }
    }
    
    
    /**
     * Removes (cancels) a lock.
     *
     * @param lock Token to remove
     * @exception ServiceAccessException Service access error
     * @exception LockTokenNotFoundException Lock token was not found
     */
    public void removeLock(Uri uri, NodeLock lock)
        throws ServiceAccessException, LockTokenNotFoundException {
        Object value = locks.get(uri.toString());
        Vector locksVector = null;
        if (value == null) {
            throw new LockTokenNotFoundException(lock);
        } else {
            locksVector = (Vector) value;
            boolean wasPresent = locksVector.removeElement(lock);
            if (!wasPresent) {
                throw new LockTokenNotFoundException(lock);
            }
        }
    }
    
    
    /**
     * Kills a lock.
     *
     * @param lock Token to remove
     * @exception ServiceAccessException Service access error
     * @exception LockTokenNotFoundException Lock token was not found
     */
    public void killLock(Uri uri, NodeLock lock)
        throws ServiceAccessException, LockTokenNotFoundException {
        removeLock(uri, lock);
    }
    
    
    /**
     * Returns the list of locks put on a subject.
     *
     * @param subject Subject
     * @return Enumeration List of locks which have been put on the subject
     * @exception ServiceAccessException Service access error
     */
    public Enumeration enumerateLocks(Uri uri)
        throws ServiceAccessException {
        Object value = locks.get(uri.toString());
        Vector locksVector = null;
        if (value == null) {
            locksVector = new Vector();
        } else {
            locksVector = (Vector) value;
        }
        return locksVector.elements();
    }
    
    
    /**
     * Retrieve a revision descriptors.
     *
     * @param uri Uri
     * @exception ServiceAccessException Service access error
     * @exception RevisionDescriptorNotFoundException Revision descriptor
     * was not found
     */
    public NodeRevisionDescriptors retrieveRevisionDescriptors(Uri uri)
        throws ServiceAccessException, RevisionDescriptorNotFoundException {
        Object tempObject = descriptors.get(uri.toString());
        if (tempObject == null) {
            throw new RevisionDescriptorNotFoundException(uri.toString());
        }
        return ((NodeRevisionDescriptors) tempObject).cloneObject();
    }
    
    
    /**
     * Create new revision descriptors.
     *
     * @param uri Uri
     * @param revisionDescriptors Node revision descriptors
     * @exception ServiceAccessException Service access error
     */
    public void createRevisionDescriptors
        (Uri uri, NodeRevisionDescriptors revisionDescriptors)
        throws ServiceAccessException {
        descriptors.put(uri.toString(), revisionDescriptors.cloneObject());
    }
    
    
    /**
     * Update revision descriptors.
     *
     * @param uri Uri
     * @param revisionDescriptors Node revision descriptors
     * @exception ServiceAccessException Service access error
     * @exception RevisionDescriptorNotFoundException Revision descriptor
     * was not found
     */
    public void storeRevisionDescriptors
        (Uri uri, NodeRevisionDescriptors revisionDescriptors)
        throws ServiceAccessException, RevisionDescriptorNotFoundException {
        if (!descriptors.containsKey(uri.toString())) {
            throw new RevisionDescriptorNotFoundException(uri.toString());
        }
        descriptors.put(uri.toString(), revisionDescriptors.cloneObject());
    }
    
    
    /**
     * Remove revision descriptors.
     *
     * @param uri Uri
     * @exception ServiceAccessException Service access error
     */
    public void removeRevisionDescriptors(Uri uri)
        throws ServiceAccessException {
        descriptors.remove(uri.toString());
    }
    
    
    /**
     * Retrieve revision descriptor.
     *
     * @param Uri uri
     * @param revisionNumber Node revision number
     */
    public NodeRevisionDescriptor retrieveRevisionDescriptor
        (Uri uri, NodeRevisionNumber revisionNumber)
        throws ServiceAccessException, RevisionDescriptorNotFoundException {
        Object result = descriptor.get(uri + "-" + revisionNumber);
        if (result == null) {
            throw new RevisionDescriptorNotFoundException(uri.toString());
        }
        return ((NodeRevisionDescriptor) result).cloneObject();
    }
    
    
    /**
     * Create new revision descriptor.
     *
     * @param uri Uri
     * @param revisionDescriptor Node revision descriptor
     * @exception ServiceAccessException Service access error
     */
    public void createRevisionDescriptor
        (Uri uri, NodeRevisionDescriptor revisionDescriptor)
        throws ServiceAccessException {
        descriptor.put(uri + "-" + revisionDescriptor.getRevisionNumber(),
                       revisionDescriptor.cloneObject());
    }
    
    
    /**
     * Update revision descriptor.
     *
     * @param uri Uri
     * @param revisionDescriptors Node revision descriptor
     * @exception ServiceAccessException Service access error
     * @exception RevisionDescriptorNotFoundException Revision descriptor
     * was not found
     */
    public void storeRevisionDescriptor
        (Uri uri, NodeRevisionDescriptor revisionDescriptor)
        throws ServiceAccessException, RevisionDescriptorNotFoundException {
        String key = uri + "-" + revisionDescriptor.getRevisionNumber();
        if (!descriptor.containsKey(key)) {
            throw new RevisionDescriptorNotFoundException(uri.toString());
        }
        descriptor.put(key, revisionDescriptor.cloneObject());
    }
    
    
    /**
     * Remove revision descriptor.
     *
     * @param uri Uri
     * @param revisionNumber Revision number
     * @exception ServiceAccessException Service access error
     */
    public void removeRevisionDescriptor(Uri uri, NodeRevisionNumber number)
        throws ServiceAccessException {
        descriptor.remove(uri + "-" + number);
    }
    
}
